@@ -10,6 +10,11 @@ from . import phplex
 from . import phpast as ast
 import ply.yacc as yacc
 
+if sys.version_info[0] == 3:
+    string_type = str
+else:
+    string_type = basestring
+
 # Get the token map
 tokens = phplex.tokens
 
@@ -1319,14 +1324,16 @@ def p_scalar_heredoc(p):
     if isinstance(p[2], ast.BinaryOp):
         # due to how lexer works, the last operation is joining an unnecessary
         # newline character
-        assert isinstance(p[2].right, str)
+        assert isinstance(p[2].right, string_type)
         p[2].right = p[2].right[:-1]
         if p[2].right:
             p[0] = p[2]
         else:
             p[0] = p[2].left
     else:
-        p[0] = p[2]
+        # due to how lexer works, the last operation is joining an unnecessary
+        # newline character
+        p[0] = p[2][:-1]
 
 def p_nowdoc(p):
     'nowdoc : START_NOWDOC nowdoc_text_content END_NOWDOC'
@@ -1564,8 +1571,17 @@ def p_encaps_list_string(p):
     if p[1] == '':
         p[0] = process_php_string_escapes(p[2])
     else:
-        p[0] = ast.BinaryOp('.', p[1], process_php_string_escapes(p[2]),
-                            lineno=p.lineno(2))
+        if isinstance(p[1], string_type):
+            # if it's only a string so far, just append the contents
+            p[0] = p[1] + process_php_string_escapes(p[2])
+        elif isinstance(p[1], ast.BinaryOp) and isinstance(p[1].right, string_type):
+            # if the last right leaf is a string, extend previous binop
+            p[0] = ast.BinaryOp('.', p[1].left, p[1].right + process_php_string_escapes(p[2]),
+                                lineno=p[1].lineno)
+        else:
+            # worst case - insert a binaryop
+            p[0] = ast.BinaryOp('.', p[1], process_php_string_escapes(p[2]),
+                                lineno=p.lineno(2))
 
 def p_encaps_var(p):
     'encaps_var : VARIABLE'
